.flag-icon(ac);
.flag-icon(arab);
.flag-icon(cefta);
.flag-icon(cp);
.flag-icon(dg);
.flag-icon(eac);
.flag-icon(es-ct);
.flag-icon(es-ga);
.flag-icon(es-pv);
.flag-icon(eu);
.flag-icon(gb-eng);
.flag-icon(gb-nir);
.flag-icon(gb-sct);
.flag-icon(gb-wls);
.flag-icon(ic);
.flag-icon(ta);
.flag-icon(un);
.flag-icon(xk);
