
.cactiPageHead {
    display: flex;
    height: 3rem;
    font-size: var(--fs-regular);
}

.menuoptions, .submenuoptions {
    height: auto !important;
    top: 34px;
    font-size: var(--fs-regular);
    border-radius: 5px;
    box-shadow: var(--box-shadow-color) 0 10px 20px;
}

.menuoptions li a, .submenuoptions li a {
    font-size: var(--fs-regular);
    padding: 4px 15px;
    text-align: unset;
}

#compact_tab_menu {
    width: 100%;
}

#compact_user_menu {
    width: 100%;
    margin-top: auto;
    margin-bottom: 20px;
    background-color: var(--background-navigation);
}

.cactiConsoleNavigationArea {
    height: calc(100vh - 4rem) !important;
    width: 15rem;
    float: left;
    background-color: var(--background-navigation);
    box-shadow: 2px 0 5px -2px rgba(0, 0, 0, 0.25);
}

