/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2024 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDtool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

/* prerequisites  */
@import url('./css/pre/fonts.css?a23648e6dc13c913f9ae2e993735631b');
@import url('./css/pre/colors.css?6a3d4dc1567013fda818848070dd9939');
@import url('./css/pre/keyframes.css?6d1d6a79b3c3bf783d25ff4a0cb83d59');

/* Cacti core */
@import url('./css/media/core.css?ad4ed321bb6b1a387743e4af1388207f') all;
@import url('./css/media/core-portrait.css?1b6db673b2a1f9c4ec954843f2e76ca5') screen and (max-width: 600px);
@import url('./css/media/core-landscape.css?e0d3497d8324f1695738d1ad40328d39') screen and (min-width: 601px);
@import url('./css/media/print.css?f8c80271650344f45629199331592924') print;

/* standard mode */
@import url('./css/media/standard.css?2458dd4d97dd94d3891b48e239ff6dd8') screen;
@import url('./css/media/standard-portrait.css?1fc97918874907735b4ed825cbfd120c') screen and (max-width: 600px);
@import url('./css/media/standard-landscape.css?857b8dce04e7cb60a0c06942c39fb51d') screen and (min-width: 601px);

/* compact mode */
@import url('./css/media/compact.css?0bda9f47e7be64d1c2709f24cd70ba00') screen;
@import url('./css/media/compact-portrait.css?675a729d0536e2b789cd45c991ad506d') screen and (max-width: 600px);
@import url('./css/media/compact-landscape.css?ce95d6c0a3041f8db65365a6d0dad1ca') screen and (min-width: 601px);

/* extensions & addons */
@import url('billboard.midwinter.css?e442c55720734e56c33de67e26a596de');
@import url('jquery-ui.midwinter.css?7c0c69ff83d0a0d9fc7d54b737c7f9d3');

/* End of file */
